# SPDX-FileCopyrightText: 2023 Susumu OTA <1632335+susumuota@users.noreply.github.com>
# SPDX-License-Identifier: MIT

from .nanoatp import BskyAgent

__version__ = "0.1.0"
__all__ = ['__version__', 'BskyAgent']



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nanoatp']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.28.2,<3.0.0']

setup_kwargs = {
    'name': 'nanoatp',
    'version': '0.1.0',
    'description': 'nano implementation of the AT Protocol (Authenticated Transfer Protocol)',
    'long_description': '# nanoatp\n\nA nano implementation of the AT Protocol (Authenticated Transfer Protocol).\n\n## Getting started\n\nFirst install the package:\n\n```bash\npip install nanoatp\n```\n\nThen in your application:\n\n```python\nfrom nanoatp import BskyAgent\n\nagent = new BskyAgent("https://bsky.social")\n```\n\n## Usage\n\n### Session management\n\nLog into a server using these APIs. You\'ll need an active session for most methods.\n\n```python\nfrom nanoatp import BskyAgent\n\nagent = new BskyAgent("https://bsky.social")\n\nagent.login(\'alice@mail.com\', \'hunter2\')\n\n# if you don\'t specify credentials, ATP_IDENTIFIER and ATP_PASSWORD environment variables will be used\n# agent.login()\n```\n\n### API calls\n\nThe agent includes methods for many common operations, including:\n\n```python\n# Feeds and content\nagent.post(record)\nagent.uploadBlob(data, contentType)\nagent.uploadImage(path, alt, contentType)\n\n# Session management\nagent.login(identifier, password)\n```\n\n## Advanced\n\n### Advanced API calls\n\nThe methods above are convenience wrappers. It covers most but not all available methods.\n\nThe AT Protocol identifies methods and records with reverse-DNS names. You can use them on the agent as well:\n\n```python\nres1 = agent.createRecord(\n    agent.session["did"],  # repo\n    "app.bsky.feed.post",  # collection\n    {\n        "$type": "app.bsky.feed.post",  # record\n        "text": "Hello, world!",\n        "createdAt": datetime.datetime.now(datetime.timezone.utc).isoformat().replace("+00:00", "Z")\n    }\n)\n```\n\n## Development\n\n```bash\nexport ATP_IDENTIFIER="foo.bsky.social"\nexport ATP_PASSWORD="password"\npoetry install\npoetry run pytest -s     # run pytest once\npoetry run -- ptw -- -s  # run pytest and watch for changes\n```\n\n## License\n\nMIT License. See [LICENSE](LICENSE) for details.\n\n## Author\n\nSusumu Ota\n',
    'author': 'Susumu OTA',
    'author_email': '1632335+susumuota@users.noreply.github.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9.16,<4.0.0',
}


setup(**setup_kwargs)



# SPDX-FileCopyrightText: 2023 Susumu OTA <1632335+susumuota@users.noreply.github.com>
# SPDX-License-Identifier: MIT

from __future__ import annotations

import datetime
from mimetypes import guess_type
import os
from typing import Any

import requests


# TODO: replace Any
class BskyAgent:
    def __init__(self, service: str = "https://bsky.social") -> None:
        self.service = service
        self.requests = requests.Session()
        self.session = None
        self.headers = {}

    def login(self, identifier: str = "", password: str = "") -> Any:
        """https://github.com/bluesky-social/atproto/blob/main/packages/api/src/agent.ts"""
        id = identifier or os.getenv("ATP_IDENTIFIER") or ""
        pw = password or os.getenv("ATP_PASSWORD") or ""
        session = self.createSession(id, pw)
        if not self._verifySession(session):
            self.session = None
            self.headers = {}
            raise Exception(str(session))
        self.session = session
        accessJwt = self.session["accessJwt"]
        self.headers = {"Authorization": f"Bearer {accessJwt}"}
        return self.session

    def post(self, record: dict[str, Any]) -> Any:
        """https://github.com/bluesky-social/atproto/blob/main/packages/api/src/bsky-agent.ts"""
        if not self.session:
            raise Exception("Not logged in")
        if not record.get("createdAt"):
            record["createdAt"] = self._now()
        if not record.get("$type"):
            record["$type"] = "app.bsky.feed.post"
        return self.createRecord(self.session["did"], "app.bsky.feed.post", record)

    def uploadImage(self, path: str, alt: str = "", contentType: str = "") -> Any:
        """https://github.com/bluesky-social/atproto/blob/main/lexicons/app/bsky/embed/images.json"""
        if not self.session:
            raise Exception("Not logged in")
        contentType = contentType or guess_type(path)[0] or "application/octet-stream"
        data = b""
        with open(path, "rb") as f:
            data = f.read()
        response = self.uploadBlob(data, contentType)
        if not response.get("blob"):
            raise Exception(str(response))
        blob = response.get("blob")
        image = {
            "$type": "app.bsky.embed.images#image",
            "alt": alt,
            "image": {
                "$type": "blob",
                "ref": blob.get("ref"),
                "mimeType": blob.get("mimeType"),
                "size": blob.get("size"),
            },
        }
        return image

    def createSession(self, identifier: str, password: str) -> Any:
        """https://github.com/bluesky-social/atproto/blob/main/lexicons/com/atproto/server/createSession.json"""
        json = {"identifier": identifier, "password": password}
        response = self.requests.post(f"{self.service}/xrpc/com.atproto.server.createSession", json=json)
        return response.json()

    def createRecord(self, repo: str, collection: str, record: dict[str, Any]) -> Any:
        """https://github.com/bluesky-social/atproto/blob/main/lexicons/com/atproto/repo/createRecord.json"""
        json = {"repo": repo, "collection": collection, "record": record}
        response = self.requests.post(
            f"{self.service}/xrpc/com.atproto.repo.createRecord", headers=self.headers, json=json
        )
        return response.json()

    def uploadBlob(self, data: bytes, contentType: str) -> Any:
        """https://github.com/bluesky-social/atproto/blob/main/lexicons/com/atproto/repo/uploadBlob.json"""
        headers = {"Content-Type": contentType, "Content-Length": str(len(data))}
        headers.update(self.headers)
        response = self.requests.post(f"{self.service}/xrpc/com.atproto.repo.uploadBlob", headers=headers, data=data)
        return response.json()

    def _verifySession(self, json: dict[str, str]) -> bool:
        return not json.get("error") and list(json.keys()) == ["did", "handle", "email", "accessJwt", "refreshJwt"]

    def _now(self) -> str:
        return datetime.datetime.now(datetime.timezone.utc).isoformat().replace("+00:00", "Z")
